@@ -346,7 +346,7 @@ class AnalysisUtils:
             fig.suptitle(title)
 
         row = 0
-        for name, group in grouped:
+        for _, group in grouped:
             group.plot.scatter('time', 'reward', ax=axes[row, 0])
             group.plot.scatter('time', 'new_reward', ax=axes[row, 1])
             group.hist(column=['time'], bins=20, ax=axes[row, 2])
@@ -496,6 +496,8 @@ class PlottingUtils:
 
             ax = fig.add_subplot(n_laps, 1, i + 1)
 
+            ax.axis('equal')
+
             PlottingUtils.print_border(ax, track, color='cyan')
 
             data_to_plot.plot.scatter('x', 'y', ax=ax, s=10, c='blue')
@@ -514,7 +516,7 @@ class PlottingUtils:
         streams = evaluations.sort_values(
             'timestamp', ascending=False).groupby('stream', sort=False)
 
-        for name, stream in streams:
+        for _, stream in streams:
             episodes = stream.groupby('episode')
             ep_count = len(episodes)
 
@@ -598,10 +600,10 @@ class PlottingUtils:
                 plt.clf()
 
     @staticmethod
-    def plot_track(df, track: Track, value_field="reward", margin=1):
+    def plot_track(df, track: Track, value_field="reward", margin=1, cmap="hot"):
         """Plot track with dots presenting the rewards for steps
         """
-        track_size = ((np.asarray(track.size()) + 2*margin)*100).astype(int)
+        track_size = (np.asarray(track.size()) + 2*margin).astype(int) * 100
         track_img = np.zeros(track_size).transpose()
 
         x_coord = 0
@@ -611,7 +613,7 @@ class PlottingUtils:
         x_compensation = df['x'].min()
         y_compensation = df['y'].min()
 
-        for index, row in df.iterrows():
+        for _, row in df.iterrows():
             x = int((row["x"] - x_compensation + margin) * 100)
             y = int((row["y"] - y_compensation + margin) * 100)
 
@@ -633,25 +635,27 @@ class PlottingUtils:
         PlottingUtils.print_border(ax, shifted_track)
 
         plt.title("Reward distribution for all actions ")
-        im = plt.imshow(track_img, cmap='hot', interpolation='bilinear', origin="lower")
+        plt.imshow(track_img, cmap=cmap, interpolation='bilinear', origin="lower")
 
         plt.show()
         plt.clf()
 
     @staticmethod
-    def plot_trackpoints(track: Track):
-        fig, ax = plt.subplots(figsize=(20, 10))
-        PlottingUtils.plot_points(ax, track.center_line)
-        PlottingUtils.plot_points(ax, track.inner_border)
-        PlottingUtils.plot_points(ax, track.outer_border)
+    def plot_trackpoints(track: Track, annotate_every_nth=1):
+        _, ax = plt.subplots(figsize=(20, 10))
+        PlottingUtils.plot_points(ax, track.center_line, annotate_every_nth)
+        PlottingUtils.plot_points(ax, track.inner_border, annotate_every_nth)
+        PlottingUtils.plot_points(ax, track.outer_border, annotate_every_nth)
         ax.axis('equal')
-        plt.show()
+
+        return ax
 
     @staticmethod
-    def plot_points(ax, points):
+    def plot_points(ax, points, annotate_every_nth=1):
         ax.scatter(points[:-1, 0], points[:-1, 1], s=1)
         for i, p in enumerate(points):
-            ax.annotate(i, (p[0], p[1]))
+            if i % annotate_every_nth == 0:
+                ax.annotate(i, (p[0], p[1]))
 
     @staticmethod
     def _plot_coords(ax, ob):
@@ -718,7 +722,7 @@ class EvaluationUtils:
                                     log_group, not_older_than, older_than)
 
         EvaluationUtils.analyse_multiple_race_evaluations(
-            logs, l_inner_border, l_outer_border, min_progress=min_progress)
+            logs, track, min_progress=min_progress)
 
 
 class NewRewardUtils:
@@ -836,7 +840,7 @@ class NewRewardUtils:
         reward = rf.Reward(verbose=verbose)
 
         new_rewards = []
-        for index, row in panda.iterrows():
+        for _, row in panda.iterrows():
             new_rewards.append(
                 reward.reward_function(NewRewardUtils.df_to_params(row, center_line)))
 
@@ -878,14 +882,21 @@ class ActionBreakdownUtils:
         return 0
 
     @staticmethod
-    def action_breakdown(df, iteration_ids, track: Track, track_breakdown=None,
-                         action_names=None):
+    def action_breakdown(
+        df,
+        track: Track,
+        iteration_ids=None,
+        episode_ids=None,
+        track_breakdown=None,
+        action_names=None,
+        min_reward=0.0
+    ):
         """Visualise action breakdown for the simulation data
 
         Arguments:
         df - dataframe to visualise
-        iteration_ids - which episodes to visualise
         track - track info to plot
+        iteration_ids - which episodes to visualise
         track_breakdown - interesting sections of the track to show,
             default: None (no sections highlighted)
         action_names - how to call the actions; default: None (names will be generated)
@@ -895,68 +906,70 @@ class ActionBreakdownUtils:
 
         fig = plt.figure(figsize=(16, len(action_names)*6))
 
-        if type(iteration_ids) is not list:
+        if iteration_ids is not None and type(iteration_ids) is not list:
             iteration_ids = [iteration_ids]
 
+        if episode_ids is not None and type(episode_ids) is not list:
+            episode_ids = [episode_ids]
+
         wpts_array = track.center_line
 
-        for iter_num in iteration_ids:
-            # Slice the data frame to get all episodes in that iteration
-            df_iter = df[(iter_num == df['iteration'])]
-            n_steps_in_iter = len(df_iter)
-            print('Number of steps in iteration=', n_steps_in_iter)
+        # Slice the data frame to get all episodes in selected iterations
+        df_iter = df[df['iteration'].isin(iteration_ids)] if iteration_ids is not None else df
 
-            th = 0.8
-            for idx in range(len(action_names)):
-                ax = fig.add_subplot(len(action_names), 2, 2 * idx + 1)
-                PlottingUtils.print_border(
+        # Slice the data frame to get all episodes in list
+        df_iter = df[df['episode'].isin(episode_ids)] if episode_ids is not None else df
+
+        for idx in range(len(action_names)):
+            ax = fig.add_subplot(len(action_names), 2, 2 * idx + 1)
+            PlottingUtils.print_border(
+                ax,
+                track
+            )
+
+            df_slice = df_iter[df_iter['action'] == idx]
+            df_slice = df_slice[df_slice['reward'] >= min_reward]
+
+            ax.plot(df_slice['x'], df_slice['y'], 'b.')
+
+            if track_breakdown:
+                for idWp in track_breakdown.vert_lines:
+                    ax.text(wpts_array[idWp][0],
+                            wpts_array[idWp][1] + 0.2,
+                            str(idWp),
+                            bbox=dict(facecolor='red', alpha=0.5))
+
+            # ax.set_title(str(log_name_id) + '-' + str(iter_num) + ' w rew >= '+str(th))
+            ax.set_ylabel(action_names[idx])
+
+            # calculate action way point distribution
+            action_waypoint_distribution = list()
+            for idWp in range(len(wpts_array)):
+                action_waypoint_distribution.append(
+                    len(df_slice[df_slice['closest_waypoint'] == idWp]))
+
+            ax = fig.add_subplot(len(action_names), 2, 2 * idx + 2)
+
+            if track_breakdown:
+                # Call function to create error boxes
+                _ = ActionBreakdownUtils._make_error_boxes(
                     ax,
-                    track
+                    track_breakdown.segment_x,
+                    track_breakdown.segment_y,
+                    track_breakdown.segment_xerr,
+                    track_breakdown.segment_yerr
                 )
 
-                df_slice = df_iter[df_iter['reward'] >= th]
-                df_slice = df_slice[df_slice['action'] == idx]
+                for tt in range(len(track_breakdown.track_segments)):
+                    ax.text(track_breakdown.track_segments[tt][0],
+                            track_breakdown.track_segments[tt][1],
+                            track_breakdown.track_segments[tt][2])
 
-                ax.plot(df_slice['x'], df_slice['y'], 'b.')
-
-                if track_breakdown:
-                    for idWp in track_breakdown.vert_lines:
-                        ax.text(wpts_array[idWp][0],
-                                wpts_array[idWp][1] + 0.2,
-                                str(idWp),
-                                bbox=dict(facecolor='red', alpha=0.5))
-
-                # ax.set_title(str(log_name_id) + '-' + str(iter_num) + ' w rew >= '+str(th))
-                ax.set_ylabel(action_names[idx])
-
-                # calculate action way point distribution
-                action_waypoint_distribution = list()
-                for idWp in range(len(wpts_array)):
-                    action_waypoint_distribution.append(
-                        len(df_slice[df_slice['closest_waypoint'] == idWp]))
-
-                ax = fig.add_subplot(len(action_names), 2, 2 * idx + 2)
-
-                if track_breakdown:
-                    # Call function to create error boxes
-                    _ = ActionBreakdownUtils._make_error_boxes(
-                        ax,
-                        track_breakdown.segment_x,
-                        track_breakdown.segment_y,
-                        track_breakdown.segment_xerr,
-                        track_breakdown.segment_yerr
-                    )
-
-                    for tt in range(len(track_breakdown.track_segments)):
-                        ax.text(track_breakdown.track_segments[tt][0],
-                                track_breakdown.track_segments[tt][1],
-                                track_breakdown.track_segments[tt][2])
-
-                ax.bar(np.arange(len(wpts_array)), action_waypoint_distribution)
-                ax.set_xlabel('waypoint')
-                ax.set_ylabel('# of actions')
-                ax.legend([action_names[idx]])
-                ax.set_ylim((0, 150))
+            ax.bar(np.arange(len(wpts_array)), action_waypoint_distribution)
+            ax.set_xlabel('waypoint')
+            ax.set_ylabel('# of actions')
+            ax.legend([action_names[idx]])
+            ax.set_ylim((0, 150))
 
         plt.show()
         plt.clf()
